import { generateAttrs, generateUsers } from "../test-utils/test-attrsUtils";
import registerMessageInApp, { roomType } from "../../mails/saveMessage";
import { jest, describe, it, expect } from "@jest/globals";

import { getAddresseId, getUserIdOfMailbox } from "../../db/mail";
// todo esbuild
// todo mock db

// new message from us
//     to multiple people -> room
//         if response has same member => group
//         if response is dm => channel
//     to one person => dm

// new message from other
//     to only me -> room
//         if no reply to multiple message => channel
//         else => dm
//     to multiple people -> room
//         // make it better
//         if multiple members reply -> group
//         if only me reply -> channel

const users = generateUsers(5);
const ownUser = users[0];
const messageId = 1;
const boxId = 1;

jest.mock("../../db/mail", () => ({
    getAddresseId: jest.fn().mockImplementation((email) => {
        const match = users.find((user) => user.user.mailbox + "@" + user.user.host == email);
        return new Promise((resolve, reject) => resolve(match?.id));
    }),
    getUserIdOfMailbox: jest.fn().mockImplementation((boxId) => {
        return new Promise((resolve, reject) => resolve([{ user_id: ownUser.id }]));
    }),
}));

describe("saveMessage", () => {
    describe("functions", () => {
        it("isFromUs", async () => {
            const attrs = generateAttrs({ from: [ownUser.user], to: [users[1].user] });
            const register = new registerMessageInApp(messageId, attrs, boxId);
            await register.init();
            const res = await register.isFromUs();

            expect(res).toBe(true);

            const attrs2 = generateAttrs({ from: [users[2].user], to: [users[1].user] });
            const register2 = new registerMessageInApp(messageId, attrs2, boxId);
            await register2.init();
            const res2 = await register2.isFromUs();

            expect(res2).toBe(false);
        });
    });
    describe("implementation", () => {
        describe("new first message from us", () => {
            it("new first message from us to one recipient should create a DM", async () => {
                const attrs = generateAttrs({ from: [ownUser.user], to: [users[1].user] });

                const register = new registerMessageInApp(messageId, attrs, boxId);

                const createOrRegisterOnExistence = jest
                    .spyOn(register, "createOrRegisterOnExistence")
                    .mockImplementation(
                        (owner: number, roomType: number) => new Promise((resolve, reject) => resolve()),
                    );

                await register.save();

                expect(createOrRegisterOnExistence).toHaveBeenCalledWith(users[1].id, roomType.DM);
            });
            it("new first message from us to multiple recipients should create a ROOM", async () => {
                const attrs = generateAttrs({ from: [ownUser.user], to: [users[1].user, users[2].user] });

                const register = new registerMessageInApp(messageId, attrs, boxId);

                const initiateRoom = jest
                    .spyOn(register, "initiateRoom")
                    .mockImplementation((owner: number, roomType: number) => Promise.resolve(1));

                await register.save();

                expect(initiateRoom).toHaveBeenCalledWith(ownUser.id, roomType.ROOM);
            });
            // it("response to new first message to multiple recipients with same members should change room type to GROUP", () => {});
            // it("response to new first message to multiple recipients with different members should change room type to CHANNEL", () => {});
        });
        describe("new first message from other", () => {
            it("new first message from other to me only should create a room", async () => {
                const attrs = generateAttrs({ from: [users[1].user], to: [ownUser.user] });

                const register = new registerMessageInApp(messageId, attrs, boxId);

                const createOrRegisterOnExistence = jest
                    .spyOn(register, "createOrRegisterOnExistence")
                    .mockImplementation((owner: number, roomType: number) => {
                        return new Promise((resolve, reject) => resolve());
                    });

                await register.save();

                expect(createOrRegisterOnExistence).toHaveBeenCalledWith(users[1].id, roomType.ROOM);
            });
        });
        // describe("replies", () => {
        //     it("", () => {});
        // });
        // describe("", () => {});
    });
});
